!
! F77-ized version of a Fortran90 TVD split MHD code
!
!  Original F90 code  copyright (C) 2001,2003, Ue-Li Pen
!  written November 2001 by Ue-Li Pen, pen@cita.utoronto.ca
!
! see http://arxiv.org/abs/astro-ph/0305088
! or http://www.cita.utoronto.ca/~pen/MHD
!
! This code is licencensed under the GPL
!
program hydro
  use pbm, only : outputpbm

  implicit none
  integer, parameter :: n=100,nx=n,ny=n
  real  :: t,dt
  integer :: iter
  real, external :: cfl
  integer :: idens, iener, imomx, imomy, nvars
  real :: gamma
  data idens/1/, iener/4/, imomx/2/, imomy/3/, nvars/4/, gamma/1.66666667/
  common /consts/ idens, iener, imomx, imomy, gamma, nvars
  real, dimension(4,nx,ny) :: u, ut

  call initialconditions(u,nx,ny)
  call outputpbm(u,nvars,nx,ny,'ics.pbm',varnum=idens)
  t=0
  timesteps: do iter=1,nx*6
    dt=0.5*cfl(u,nx,ny)

! the x sweep
    call xsweep(u,nx,ny,dt)

! the y sweeps
    call xytranspose(ut,u,nx,ny)
    call xsweep(ut,ny,nx,dt)
    t = t + dt
    call xsweep(ut,ny,nx,dt)
! 2nd x sweep
    call xytranspose(u,ut,ny,nx)
    call xsweep(u,nx,ny,dt)
    t = t + dt
    if (mod(iter,10).eq.1) print *, iter, 'dt = ', dt, ' t = ', t

   end do timesteps
   call outputpbm(u,nvars,nx,ny,filename='dens.pbm')
end

 real function cfl(u,nx,ny)
   implicit none
   integer :: nx, ny
   real, dimension(nvars, nx, ny) :: u
   integer :: idens, iener, imomx, imomy, nvars
   real :: gamma
   common /consts/ idens, iener, imomx, imomy, gamma, nvars
   real :: v,p,c
   integer :: i,j
   
   c = 0.
   do j=1,ny
      do i=1,nx
         v=max(abs(u(imomx,i,j)),abs(u(imomy,i,j)))/u(idens,i,j)
         p=(u(iener,i,j)-(u(imomx,i,j)**2+u(imomy,i,j)**2)/ &
                         u(idens,i,j)/2)*(gamma-1)
         c=max(c,v+sqrt(abs((gamma*p)/u(idens,i,j))))
      enddo
   enddo
   cfl=1./c
 end function cfl

subroutine initialconditions(u,nx,ny)
  implicit none
  integer :: idens, iener, imomx, imomy, nvars
  real    :: gamma
  common /consts/ idens, iener, imomx, imomy, gamma, nvars
  integer :: nx,ny
  real    :: u(nvars,nx,ny)
  integer :: i,j
  real, parameter :: backgrounddens=1., projdens=50., projvel=4.
  real :: r

  xloop: do i=1,nx
    yloop: do j=1,ny
      u(imomy,i,j)=0
      r = sqrt((j*1.0-ny/2.)**2 + (i*1.0-nx/2.)**2)
      if (r < 0.1*sqrt(nx*nx*1.+ny*ny*1.)) then
          u(idens,i,j) = projdens
          u(imomx,i,j) = projdens*projvel
          u(iener,i,j) =  0.5*(projdens*projvel*projvel) + (1./((gamma-1.)*projdens))
      else
          u(idens,i,j) = backgrounddens
          u(imomx,i,j) = backgrounddens*0.
          u(iener,i,j) =  0.5*(backgrounddens*0.*0.) + &
                          (1./((gamma-1.)*backgrounddens))
      endif
    enddo yloop
  enddo xloop
  return
end subroutine initialconditions

subroutine xsweep(u,nx,ny,dt)
  implicit none
  integer :: idens, iener, imomx, imomy, nvars
  real :: gamma
  common /consts/ idens, iener, imomx, imomy, gamma, nvars
  integer :: nx,ny
  real, dimension(nvars,nx,ny) ::  u
  real :: dt
  integer :: j

  do j=1,ny
     call tvd1d(u(1,1,j),nx,dt)
  enddo
end subroutine xsweep

subroutine tvd1d(u,n,dt)
  implicit none
  integer :: idens, iener, imomx, imomy, nvars
  real :: gamma
  common /consts/ idens, iener, imomx, imomy, gamma, nvars
  integer :: n
  real :: dt
  real, dimension(nvars,n) :: u

  real, dimension(nvars,n) :: v, u1, wr, wl
  real :: dfrp, dfrm, dflm, dflp
  real :: dfl(nvars,n), dfr(nvars,n)
  real :: flux(nvars,n)
  real :: c
  integer :: i, var
  integer :: ilshift, irshift, irrshift

  call hydroflux(v,c,u,n)
  do i=1,n
      do var=1,nvars
          wr(var,i)=u(var,i)+v(var,i)
          wl(var,i)=u(var,i)-v(var,i)
      enddo
  enddo
  do i=1,n
      irshift=mod(n+(i-1)+1,n)+1
      do var=1,nvars
          flux(var,i)=c*(wr(var,i)-wl(var,irshift))/2.
      enddo
  enddo
  do i=1,n
      ilshift=mod(n+(i-1)-1,n)+1
      do var=1,nvars
          u1(var,i)=u(var,i)-(flux(var,i)-flux(var,ilshift))*dt/2.
      enddo
  enddo

  call hydroflux(v,c,u1,n)
  do i=1,n
      do var=1,nvars
          wr(var,i)=u1(var,i)+v(var,i)
          wl(var,i)=u1(var,i)-v(var,i)
      enddo
  enddo
  do i=1,n
      irshift=mod(n+(i-1)+1,n)+1
      ilshift=mod(n+(i-1)-1,n)+1
      do var=1,nvars
          dfrp = c*(wr(var,irshift)-wr(var,i))/2.
          dfrm = c*(wr(var,i)-wr(var,ilshift))/2.
          dfr(var,i) =0.
          if (dfrp*dfrm > 0) then
             dfr(var,i)=2*dfrp*dfrm/(dfrp+dfrm)
          endif
      enddo
  enddo
  do i=1,n
      irshift=mod(n+(i-1)+1,n)+1
      irrshift=mod(n+(i-1)+2,n)+1
      do var=1,nvars
          dflp = c*(wl(var,irshift)-wl(var,irrshift))/2.
          dflm = c*(wl(var,i)-wl(var,irshift))/2.
          dfl(var,i) =0.
          if (dflp*dflm > 0) then
             dfl(var,i)=2*dflp*dflm/(dflp+dflm)
          endif
      enddo
  enddo
  do i=1,n
     irshift=mod(n+(i-1)+1,n)+1
     do var=1,nvars
         flux(var,i)=(c*wr(var,i)-c*wl(var,irshift)+ &
                    (dfr(var,i)-dfl(var,i)))/2.
     enddo
  enddo
  do i=1,n
     ilshift=mod(n+(i-1)-1,n)+1
     do var=1,nvars
        u(var,i)=u(var,i)-(flux(var,i)-flux(var,ilshift))*dt
     enddo
  enddo
  return
end subroutine tvd1d

subroutine hydroflux(v,c,u,n)
  implicit none
  integer :: n
  real    :: c
  real, dimension(n) :: vx, p
  integer :: idens, iener, imomx, imomy, nvars
  real    ::gamma
  common /consts/ idens, iener, imomx, imomy, gamma, nvars
  real, dimension(nvars,n) :: v, u
  integer :: i

  c = 0.
  do i=1,n
    vx(i)=u(imomx,i)/u(idens,i)
    p(i)=u(iener,i)-(u(imomx,i)**2+u(imomy,i)**2)/u(idens,i)/2
    p(i)=p(i)*(gamma-1)
    v(idens,i)=u(imomx,i)
    v(imomx,i)=u(imomx,i)*vx(i)+p(i)
    v(imomy,i)=u(imomy,i)*vx(i)
    v(iener,i)=(u(iener,i)+p(i))*vx(i)
    c=max(c, abs(vx(i))+sqrt(abs(gamma*p(i)/u(idens,i))))
  enddo
  if (c > 0) then
     do i=1,n
        v(idens,i) = v(idens,i)/c
        v(imomx,i) = v(imomx,i)/c
        v(imomy,i) = v(imomy,i)/c
        v(iener,i) = v(iener,i)/c
     enddo
  endif
end subroutine hydroflux

subroutine xytranspose(ut,u,nx,ny)
  implicit none
  integer :: idens, iener, imomx, imomy, nvars
  real :: gamma
  common /consts/ idens, iener, imomx, imomy, gamma, nvars
  integer :: nx,ny
  real, dimension(nvars,nx,ny) :: u, ut
  integer :: i,j

  do j=1,ny
     do i=1,nx
        ut(idens,j,i)=u(idens,i,j)
        ut(imomx,j,i)=u(imomy,i,j)
        ut(imomy,j,i)=u(imomx,i,j)
        ut(iener,j,i)=u(iener,i,j)
     enddo
  enddo
end subroutine xytranspose
